@@ -41,11 +41,11 @@ ko2kegg_abundance <- function (file = NULL, data = NULL) {
   if (is.null(file) & is.null(data)) {
     stop("Error: Please provide either a file or a data.frame.")
   }
-  
+
   if (!is.null(file) && !is.null(data)) {
     warning("Both file and data provided. Using data and ignoring file.")
   }
-  
+
   # 文件格式验证函数
   validate_file_format <- function(file_path) {
     valid_extensions <- c(".txt", ".tsv", ".csv")
@@ -55,7 +55,7 @@ ko2kegg_abundance <- function (file = NULL, data = NULL) {
                   paste(valid_extensions, collapse = ", ")))
     }
   }
-  
+
   # 数据框验证函数
   validate_abundance_data <- function(df) {
     if (ncol(df) < 2) {
@@ -68,21 +68,21 @@ ko2kegg_abundance <- function (file = NULL, data = NULL) {
       stop("Error: All sample columns must contain numeric values.")
     }
   }
-  
+
   # 添加新的输入验证函数
   validate_input <- function(data) {
     # 基本结构验证
     if (!is.data.frame(data)) {
       stop("The provided data must be a data.frame")
     }
-    
+
     if (ncol(data) < 2) {
       stop("Data must contain at least one KO column and one sample column")
     }
-    
+
     # 检查第一列（KO IDs）
     ko_col <- data[[1]]
-    
+
     # 检查 KO ID 格式
     ko_pattern <- "^K\\d{5}$"
     invalid_kos <- ko_col[!grepl(ko_pattern, ko_col)]
@@ -93,10 +93,10 @@ ko2kegg_abundance <- function (file = NULL, data = NULL) {
         paste(head(invalid_kos, 3), collapse = ", ")
       ))
     }
-    
+
     # 检查数值列
     numeric_cols <- data[,-1, drop = FALSE]
-    
+
     # 检查是否所有样本列都是数值型
     non_numeric_cols <- names(numeric_cols)[!sapply(numeric_cols, is.numeric)]
     if (length(non_numeric_cols) > 0) {
@@ -105,7 +105,7 @@ ko2kegg_abundance <- function (file = NULL, data = NULL) {
         paste(non_numeric_cols, collapse = ", ")
       ))
     }
-    
+
     # 检查负值
     neg_values <- sapply(numeric_cols, function(x) any(x < 0, na.rm = TRUE))
     if (any(neg_values)) {
@@ -114,7 +114,7 @@ ko2kegg_abundance <- function (file = NULL, data = NULL) {
         paste(names(numeric_cols)[neg_values], collapse = ", ")
       ))
     }
-    
+
     # 检查缺失值
     na_counts <- sapply(numeric_cols, function(x) sum(is.na(x)))
     cols_with_na <- names(na_counts[na_counts > 0])
@@ -125,7 +125,7 @@ ko2kegg_abundance <- function (file = NULL, data = NULL) {
         paste(paste(cols_with_na, na_counts[cols_with_na], sep = ": "), collapse = ", ")
       ))
     }
-    
+
     # 检查全零行
     zero_rows <- rowSums(numeric_cols == 0) == ncol(numeric_cols)
     if (any(zero_rows)) {
@@ -134,15 +134,15 @@ ko2kegg_abundance <- function (file = NULL, data = NULL) {
         sum(zero_rows)
       ))
     }
-    
+
     # 检查列名是否唯一
     if (any(duplicated(colnames(data)))) {
       stop("Duplicate column names found in the input data")
     }
-    
+
     return(TRUE)
   }
-  
+
   # 文件格式验证
   if (!is.null(file)) {
     validate_file_format(file)
@@ -158,7 +158,7 @@ ko2kegg_abundance <- function (file = NULL, data = NULL) {
     message("Processing provided data frame...")
     abundance <- data
   }
-  
+
   # 运行输入验证
   tryCatch({
     validate_input(abundance)
@@ -167,23 +167,23 @@ ko2kegg_abundance <- function (file = NULL, data = NULL) {
   }, error = function(e) {
     stop("Data validation failed: ", e$message, call. = FALSE)
   })
-  
+
   message("Loading KEGG reference data. This might take a while...")
   if (!exists("ko_to_kegg_reference")) {
     data("ko_to_kegg_reference", package = "ggpicrust2", envir = environment())
   }
-  
+
   # 转换参考数据格式
   ko_to_kegg_reference <- as.data.frame(ko_to_kegg_reference)
   kegg_names <- ko_to_kegg_reference[, 1]  # 第一列是KEGG pathway IDs
-  
+
   message(sprintf("Processing %d KEGG pathways...", length(unique(kegg_names))))
-  
+
   # 创建结果数据框
   kegg_abundance <- data.frame(
     row.names = unique(kegg_names)  # 将 pathway 设置为行名
   )
-  
+
   # 为每个样本添加列
   sample_names <- colnames(abundance)[-1]
   for (sample in sample_names) {
@@ -194,18 +194,18 @@ ko2kegg_abundance <- function (file = NULL, data = NULL) {
   tryCatch({
     pb <- txtProgressBar(min = 0, max = nrow(kegg_abundance), style = 3)
     start_time <- Sys.time()
-    
+
     total_matches <- 0
-    
+
     for (i in seq_len(nrow(kegg_abundance))) {
       setTxtProgressBar(pb, i)
       current_kegg <- rownames(kegg_abundance)[i]
-      
+
       # 获取当前KEGG pathway对应的所有KOs
       pathway_row <- ko_to_kegg_reference[ko_to_kegg_reference[,1] == current_kegg, ]
       relevant_kos <- unlist(pathway_row[,-1])  # 除第一列外的所有KO IDs
       relevant_kos <- relevant_kos[!is.na(relevant_kos) & relevant_kos != ""]
-      
+
       # 查找匹配的KOs
       matching_rows <- abundance[[1]] %in% relevant_kos
       if (any(matching_rows)) {
@@ -213,28 +213,30 @@ ko2kegg_abundance <- function (file = NULL, data = NULL) {
         kegg_abundance[i, ] <- colSums(abundance[matching_rows, -1, drop = FALSE])
       }
     }
-    
+
     end_time <- Sys.time()
     close(pb)
-    
+
     # 添加调试信息
     message(sprintf("Total KO matches found: %d", total_matches))
-    message(sprintf("Number of non-zero pathways before filtering: %d", 
+    message(sprintf("Number of non-zero pathways before filtering: %d",
                    sum(rowSums(kegg_abundance[,-1, drop = FALSE]) > 0)))
-    
+
     # 移除零丰度通路
     message("Removing KEGG pathways with zero abundance across all samples...")
     zero_rows <- rowSums(kegg_abundance[,-1, drop = FALSE]) == 0
     kegg_abundance <- kegg_abundance[!zero_rows, , drop = FALSE]
-    
+
     message(sprintf("Final number of KEGG pathways: %d", nrow(kegg_abundance)))
-    
+
     if (nrow(kegg_abundance) == 0) {
-      stop("No non-zero KEGG pathways found after processing")
+      message("No non-zero KEGG pathways found after processing")
+      # Return empty data frame instead of stopping
+      return(kegg_abundance)
     }
-    
+
     return(kegg_abundance)
-    
+
   }, error = function(e) {
     if (exists("pb")) close(pb)
     message("Error occurred during processing: ", e$message)
